@@ -5,7 +5,7 @@ import hippo, gobject, gtk, dbus, dbus.glib
 
 import bigboard.globals as globals
 from bigboard.libbig.singletonmixin import Singleton
-from bigboard.libbig.http import AsyncHTTPFetcher
+from bigboard.libbig.http import AsyncHTTPFetcherWithAuth
 from bigboard import libbig
 from bigboard.workboard import WorkBoard
 import libbig.logutil
@@ -14,7 +14,6 @@ from libbig.logutil import log_except
 from libbig.struct import AutoStruct, AutoSignallingStruct
 import libbig.polling
 import htmllib
-import bigboard.accounts as accounts
 import gdata.docs as gdocs
 
 _logger = logging.getLogger("bigboard.Google")
@@ -188,35 +187,6 @@ class NewMailParser(xml.sax.ContentHandler):
     def get_new_mails(self):
         return self.__mails
 
-class AsyncHTTPFetcherWithAuth(object):
-    """Small proxy class which expects username/password combinations for fetch(),
-    and supports invoking a callback on 401."""
-    def __init__(self):
-        super(AsyncHTTPFetcherWithAuth, self).__init__()
-        self.__fetcher = AsyncHTTPFetcher()
-        
-    def fetch(self, url, username, password, cb, errcb, authcb):
-        self.__fetcher.fetch_extended(url=url, cb=cb, 
-                                      no_store=True,
-                                      response_errcb=functools.partial(self.__handle_response_error, authcb, errcb),
-                                      setupfn=functools.partial(self.__http_setupfn, username, password))
-        
-    def __http_setupfn(self, username, password, h):
-        h.add_credentials(username, password)
-        h.follow_all_redirects = True
-        
-    def __handle_response_error(self, authcb, errcb, url, response, content):
-        if response and response.status == 401:
-            _logger.debug("auth failure for fetch of %s; response message %s; invoking auth callback" % (url, response.reason))
-            gobject.idle_add(lambda: authcb(url) and False)
-        else:
-            if response: 
-                _logger.error("caught error for fetch of %s (status %s)", url, response.status)
-            else:
-                _logger.error("caught error for fetch of %s, response is None, possibly the network connection is down", url)
-            # in my experience sys.exc_info() is some kind of junk here, while "e" is useful
-            gobject.idle_add(lambda: errcb(url, response) and False)
-
 class GooglePollAction(object):
     def __init__(self, google):
         self.__google = google
@@ -387,7 +357,7 @@ class Google(gobject.GObject):
         self.__checker = CheckGoogleTask(self)
         self.add_poll_action('mail', lambda: MailPollAction(self))
 
-        self.__account.connect('changed', lambda account: self.__on_account_changed())
+        self.__account.connect_to_signal('Changed', self.__on_account_changed)
         self.__on_account_changed()
 
     def get_account(self):
@@ -417,8 +387,8 @@ class Google(gobject.GObject):
 
     def __auth_needs_retry(self):
 
-        username = self.__account.get_username_as_google_email()
-        password = self.__account.get_password()
+        username = self.__account.GetUsername()
+        password = self.__account.GetPassword()
 
         _logger.debug("auth retry for google username %s" % (username))
 
@@ -439,14 +409,16 @@ class Google(gobject.GObject):
         errcb({ 'status' : 401, 'message' : 'Bad or missing username or password' })
 
     def get_current_auth_credentials_known_bad(self):
-        return self.__account.get_username_as_google_email() == '' or \
+        return self.__account.GetUsername() == '' or \
                self.__last_auth_attempt_failed
 
+    @log_except(_logger)
     def __on_account_changed(self):
+        _logger.debug("got Changed signal!")
         auth_changed = False
-        if self.__last_username_tried != self.__account.get_username_as_google_email():
+        if self.__last_username_tried != self.__account.GetUsername():
             auth_changed = True
-        if self.__last_password_tried != self.__account.get_password():
+        if self.__last_password_tried != self.__account.GetPassword():
             auth_changed = True
 
         _logger.debug("google account changed, auth_changed=%d" % (auth_changed))
@@ -464,12 +436,13 @@ class Google(gobject.GObject):
 
     def __have_login_fetch_calendar_list(self, cb, errcb):
 
-        username = self.__account.get_username_as_google_email()
-        password = self.__account.get_password()
+        username = self.__account.GetUsername()
+        password = self.__account.GetPassword()
 
-        # there is a chance that someone might have access to more than 25 calendars, so let's
-        # specify 1000 for max-results to make sure we get information about all calendars 
-        uri = 'http://www.google.com/calendar/feeds/' + username + '?max-results=1000'
+        # We used to specify a large max-results here to make sure we got all calendars,
+        # but max-results is no longer allowed. We hope that the feed now just returns
+        # all calendars.
+        uri = 'http://www.google.com/calendar/feeds/' + username
 
         self.__fetcher.fetch(uri, username, password,
                              lambda url, data: cb(url, data, self),
@@ -481,8 +454,8 @@ class Google(gobject.GObject):
 
     def __have_login_fetch_calendar(self, cb, errcb, calendar_feed_url, event_range_start, event_range_end):
 
-        username = self.__account.get_username_as_google_email()
-        password = self.__account.get_password()        
+        username = self.__account.GetUsername()
+        password = self.__account.GetPassword()        
 
         min_and_max_str = ""
         if event_range_start is not None and event_range_end is not None:
@@ -517,8 +490,8 @@ class Google(gobject.GObject):
         errcb(exc_info)
 
     def __have_login_fetch_documents(self, cb, errcb):
-        username = self.__account.get_username_as_google_email()
-        password = self.__account.get_password()        
+        username = self.__account.GetUsername()
+        password = self.__account.GetPassword()        
         
         uri = 'http://docs.google.com/feeds/documents/private/full'
         # uri = 'http://spreadsheets.google.com/feeds/spreadsheets/private/full'
@@ -534,7 +507,7 @@ class Google(gobject.GObject):
     ### New Mail
 
     def get_mail_base_url(self):
-        username = self.__account.get_username_as_google_email()
+        username = self.__account.GetUsername()
         
         if not username:
             return None
@@ -566,8 +539,8 @@ class Google(gobject.GObject):
 
     def __have_login_fetch_new_mail(self, cb, errcb):
 
-        username = self.__account.get_username_as_google_email()
-        password = self.__account.get_password()        
+        username = self.__account.GetUsername()
+        password = self.__account.GetPassword()        
 
         uri = self.get_mail_base_url() + '/feed/atom'
 
@@ -588,34 +561,63 @@ def get_googles():
 
 def get_google_for_account(account):
     global __googles_by_account
-    if not __googles_by_account.has_key(account):
+    if not __googles_by_account.has_key(account.GetObjectPath()):
         _logger.debug("calling refresh googles from get google for account")
-        __refresh_googles(accounts.get_accounts())
+        __refresh_googles()
+ 
+    for key in __googles_by_account.keys():
+        _logger.debug("key in __googles_by_account %s" % key)
+ 
+    return __googles_by_account[account.GetObjectPath()]
 
-    return __googles_by_account[account]
-
-def __refresh_googles(a):
+def __refresh_googles():
     _logger.debug("refresh googles was called")
-    global __googles_by_account    
-    gaccounts = a.get_accounts_with_kind(accounts.KIND_GOOGLE)
-    new_googles = {}
-    for g in gaccounts:
-        if g in __googles_by_account:
-            new_googles[g] = __googles_by_account[g]
-        else:
-            new_googles[g] = Google(g)
+    global __googles_by_account 
+    try:
+        __onlineaccounts_proxy = dbus.SessionBus().get_object('org.gnome.OnlineAccounts', '/onlineaccounts')
+    except dbus.DBusException, e:
+        _logger.error("onlineaccounts DBus service not available, can't refresh googles")   
+        return
 
-    for (g, old) in __googles_by_account.items():
-        if g not in new_googles:
+    try: 
+        __onlineaccounts_proxy.EnsureAccountType("google", "Google", "Gmail or Google Apps for Your Domain email", "http://gmail.com")
+    except dbus.DBusException, e:
+        _logger.error(e.message)
+        return
+
+    gaccount_paths = __onlineaccounts_proxy.GetEnabledAccountsWithTypes(['google'])
+    new_googles = {}
+    for g_path in gaccount_paths:
+        if g_path in __googles_by_account:
+                new_googles[g_path] = __googles_by_account[g_path]
+        else:
+            try:
+                g = dbus.SessionBus().get_object('org.gnome.OnlineAccounts', g_path)  
+                new_googles[g_path] = Google(g)
+            except dbus.DBusException, e:
+                _logger.error("onlineaccount for path %s was not found" % g_path)
+
+    for (g_path, old) in __googles_by_account.items():
+        if g_path not in new_googles:
             old.destroy()
             
     __googles_by_account = new_googles
 
-def __on_account_added(a, account):
-    __refresh_googles(a)
+def __on_account_added(account_path):
+    try:
+        a = dbus.SessionBus().get_object('org.gnome.OnlineAccounts', account_path)
+        if a.GetType() == "google":
+            __refresh_googles()      
+    except dbus.DBusException, e:
+        _logger.error("onlineaccount for path %s was not found" % account_path)
 
-def __on_account_removed(a, account):
-    __refresh_googles(a)
+def __on_account_removed(account_path):
+    try:
+        a = dbus.SessionBus().get_object('org.gnome.OnlineAccounts', account_path)
+        if a.GetType() == "google":
+            __refresh_googles()      
+    except dbus.DBusException, e:
+        _logger.error("onlineaccount for path %s was not found" % account_path)
 
 def init():
     global __initialized
@@ -624,10 +626,14 @@ def init():
         return
     __initialized = True
     _logger.debug("THE google is being initialized")
-    a = accounts.get_accounts()
-    __refresh_googles(a)
-    a.connect('account-added', __on_account_added)
-    a.connect('account-removed', __on_account_removed)
+    __refresh_googles()
+
+    try:
+        __onlineaccounts_proxy = dbus.SessionBus().get_object('org.gnome.OnlineAccounts', '/onlineaccounts')
+        __onlineaccounts_proxy.connect_to_signal('AccountEnabled', __on_account_added)
+        __onlineaccounts_proxy.connect_to_signal('AccountDisabled', __on_account_removed)
+    except dbus.DBusException, e:
+        _logger.error("onlineaccounts DBus service not available, can't connect to signals")
         
 if __name__ == '__main__':
 
@@ -646,10 +652,6 @@ if __name__ == '__main__':
     bignative.set_application_name("BigBoard")
     bignative.set_program_name("bigboard")
 
-    # to test, you have to change this temporarily to store your correct
-    # password, then comment it out again.
-    # keyring.get_keyring().store_login('google', 'hp@redhat.com', 'wrong')
-
     #AuthDialog().present('foo', 'bar')
     #gtk.main()
     #sys.exit(0)
