@@ -400,7 +400,7 @@ class SelfStock(AbstractMugshotStock):
          
     def __do_logout(self):
         self._panel.action_taken()
-        self.__get_session_manager().Logout(0)
+        self.__get_session_manager().Logout(dbus.UInt32(0))
 
     def __do_shutdown(self):
         self._panel.action_taken()
